#include "graphic.h"
#include "opengame.h"
#include <cmath>

inline int concatRGBA(int r, int g, int b, int a){
	
	r = max(r, 0);
	g = max(g, 0);
	b = max(b, 0);
	a = max(a, 0);
	
	r = min(r, 255);
	g = min(g, 255);
	b = min(b, 255);
	a = min(a, 255);

	return (r<<16) | (g<<8) | (b) | (a<<24);

}

TS_Point TS_Point::operator=(const SDL_Rect &c2)
{
    // use the Cents constructor and operator+(int, int)
    return TS_Point(c2.x, c2.y);
}

TS_Point::TS_Point(int _x, int _y){
    x = _x;
    y = _y;
}

TS_Point::~TS_Point(){
    //printf("%i is the address of a freed TS_Point.", this);
}

TS_Segment::TS_Segment(int _x1, int _y1, int _x2, int _y2){
    x1 = _x1;
    y1 = _y1;
    x2 = _x2;
    y2 = _y2;
}

TS_Segment::TS_Segment(){
}

TS_Segment::~TS_Segment(){
    //printf("%i is the address of a freed TS_Point.", this);
}

v8::Handle<v8::Value> TS_surfaceBlit(const v8::Arguments& args);
v8::Handle<v8::Value> TS_surfaceZoomBlit(const v8::Arguments& args);
v8::Handle<v8::Value> TS_surfaceStretchBlit(const v8::Arguments& args);
v8::Handle<v8::Value> TS_surfaceBlitSurface(const v8::Arguments& args);
v8::Handle<v8::Value> TS_surfaceSetPixel(const v8::Arguments& args);
v8::Handle<v8::Value> TS_surfaceGetPixel(const v8::Arguments& args);

	v8::Persistent<v8::FunctionTemplate> Colortempl;// = *v8::FunctionTemplate::New();
	v8::Persistent<v8::ObjectTemplate> ColorInsttempl;// = *Colortempl->InstanceTemplate();

void ColorInit(){
    Colortempl = *v8::FunctionTemplate::New();
	ColorInsttempl = *Colortempl->InstanceTemplate();

}

void ColorClose(){
    Colortempl.Dispose();
	ColorInsttempl.Dispose();

}

	v8::Persistent<v8::FunctionTemplate> Surfacetempl;//   = *v8::FunctionTemplate::New();
	v8::Persistent<v8::ObjectTemplate> SurfaceInsttempl;// = *Surfacetempl->InstanceTemplate();
	v8::Persistent<v8::ObjectTemplate> Surfaceproto;//    = *Surfacetempl->PrototypeTemplate();

	v8::Persistent<v8::FunctionTemplate> Imagetempl;//     = *v8::FunctionTemplate::New();
	v8::Persistent<v8::ObjectTemplate> ImageInsttempl;//   = *Imagetempl->InstanceTemplate();
//I'm going to try binding the SDL surface object directly to V8. Wish me luck!!

void SurfaceInit(){
    Surfacetempl = *v8::FunctionTemplate::New();
    SurfaceInsttempl = *Surfacetempl->InstanceTemplate();
    Surfaceproto     = *Surfacetempl->PrototypeTemplate();
	Surfaceproto->Set("blit", v8::FunctionTemplate::New(TS_surfaceBlit));
	Surfaceproto->Set("transformBlit", v8::FunctionTemplate::New(TS_surfaceBlit));
	Surfaceproto->Set("zoomBlit", v8::FunctionTemplate::New(TS_surfaceZoomBlit));
	Surfaceproto->Set("stretchBlit", v8::FunctionTemplate::New(TS_surfaceStretchBlit));
	Surfaceproto->Set("setPixel", v8::FunctionTemplate::New(TS_surfaceSetPixel));
	Surfaceproto->Set("getPixel", v8::FunctionTemplate::New(TS_surfaceGetPixel));
	Surfaceproto->Set("blitSurface", v8::FunctionTemplate::New(TS_surfaceBlitSurface));
	Surfaceproto->Set("createImage", v8::FunctionTemplate::New());
	Surfaceproto->Set("createSurface", v8::FunctionTemplate::New());
}

void SurfaceClose(){
    Surfacetempl.Dispose();
    SurfaceInsttempl.Dispose();
    Surfaceproto.Dispose();
}

void ImageInit(){
	Imagetempl     = *v8::FunctionTemplate::New();
	ImageInsttempl   = *Imagetempl->InstanceTemplate();
}

void ImageClose(){
    Imagetempl.Dispose();
    ImageInsttempl.Dispose();
}
void TS_drawAlphaRect(int x, int y, int w, int h, int c, int a, SDL_Rect rect)
{
	SDL_Surface *rectsurface = SDL_CreateRGBSurface(SDL_HWSURFACE, w, h, screen->format->BitsPerPixel, 0, 0, 0, 0);
	SDL_FillRect(rectsurface, &rect,  (c<<2)|a);
	SDL_BlitSurface(rectsurface, NULL, screen, &rect);
	SDL_FreeSurface(rectsurface);
}


//Primitive Drawing:
//Contains all Sphere 1.x primitives

v8::Handle<v8::Value> Point(const v8::Arguments& args)
{
	if(args.Length()<3){
        return v8::ThrowException(v8::String::New("TS_Point Error: Called with fewer than 3 arguments!"));
	}

    CHECK_ARG_INT(0, "TS_Point Error: Argument 0 is not a number.");
    CHECK_ARG_INT(1, "TS_Point Error: Argument 1 is not a number.");
    CHECK_ARG_OBJ(2, "TS_Point Error: Argument 2 is not an object.");
	int x = (int)args[0]->v8::Value::NumberValue();
	int y = (int)args[1]->v8::Value::NumberValue();

	v8::Handle<v8::Object> color = v8::Handle<v8::Object>::Cast(args[2]);
    TS_Color* c = (TS_Color*)color->GetPointerFromInternalField(0);

    SDL_Rect rect = {(short int)x, (short int)y, 1, 1};
    SDL_FillRect(screen, &rect, c->toInt());

	return v8::Undefined();
}

v8::Handle<v8::Value> FilledCircle(const v8::Arguments& args)
{
	if(args.Length()<4){
	return v8::ThrowException(v8::String::New("TS_FilledCircle Error: Called with fewer than 4 arguments!"));
	}

    CHECK_ARG_INT(0, "TS_FilledCircle Error: argument 0 is not a valid number.");
    CHECK_ARG_INT(1, "TS_FilledCircle Error: argument 1 is not a valid number.");
    CHECK_ARG_INT(2, "TS_FilledCircle Error: argument 2 is not a valid number.");
    CHECK_ARG_OBJ(3, "TS_FilledCircle Error: argument 3 is not an object.");

	int x = (int)args[0]->v8::Value::NumberValue();
	int y = (int)args[1]->v8::Value::NumberValue();
	int rad = (int)args[2]->v8::Value::NumberValue();
	v8::Local<v8::Object> color = v8::Local<v8::Object>::Cast(args[3]);
	Uint8 r = color->Get(v8::String::New("red"))->v8::Value::IntegerValue();
	Uint8 g = color->Get(v8::String::New("green"))->v8::Value::IntegerValue();
	Uint8 b = color->Get(v8::String::New("blue"))->v8::Value::IntegerValue();
	Uint8 a = color->Get(v8::String::New("alpha"))->v8::Value::IntegerValue();

	//color.Clear();

	TS_FilledCircle(x, y, rad, r, g, b, a);

	return v8::Undefined();
}


v8::Handle<v8::Value> OutlinedCircle(const v8::Arguments& args)
{

	int x = args[0]->v8::Value::IntegerValue();
	int y = args[1]->v8::Value::IntegerValue();
	int rad = args[2]->v8::Value::IntegerValue();
	v8::Local<v8::Object> color = v8::Local<v8::Object>::Cast(args[3]);

	Uint8 r = (Uint8)color->Get(v8::String::New("red"))->v8::Value::Int32Value();
	Uint8 g = (Uint8)color->Get(v8::String::New("green"))->v8::Value::Int32Value();
	Uint8 b = (Uint8)color->Get(v8::String::New("blue"))->v8::Value::Int32Value();
	Uint8 a = (Uint8)color->Get(v8::String::New("alpha"))->v8::Value::Int32Value();

    color.Clear();

	TS_OutlinedCircle(x, y, rad, r, g, b, a);

	return v8::Undefined();
}

SDL_Surface *TS_SurfaceFromSpec(int w, int h, int color, bool hwsurface){

    SDL_Surface *surface = SDL_CreateRGBSurface((hwsurface?SDL_HWSURFACE:SDL_SWSURFACE)|SDL_SRCALPHA, w, h, DEPTH, CHANNEL_MASKS);
	SDL_Rect rect = {0, 0, (short unsigned int)w, (short unsigned int)h};
	SDL_FillRect(surface, &rect, color);
    return surface;
}


v8::Handle<v8::Value> Rectangle(const v8::Arguments& args)
{
    if(args.Length()<5){
        return v8::ThrowException(v8::String::New("TS_Rectangle Error: Called with fewer than 5 arguments!"));
	}
    CHECK_ARG_INT(0, "TS_Rectangle Error: Arg 0 is not a number.");
    CHECK_ARG_INT(1, "TS_Rectangle Error: Arg 1 is not a number.");
    CHECK_ARG_INT(2, "TS_Rectangle Error: Arg 2 is not a number.");
    CHECK_ARG_INT(3, "TS_Rectangle Error: Arg 3 is not a number.");
    CHECK_ARG_OBJ(4, "TS_Rectangle Error: Arg 4 is not an object.");
	int x = args[0]->v8::Value::IntegerValue();
	int y = args[1]->v8::Value::IntegerValue();
	int w = args[2]->v8::Value::IntegerValue();
	int h = args[3]->v8::Value::IntegerValue();

	v8::Local<v8::Object> color = v8::Local<v8::Object>::Cast(args[4]);

    TS_Color* c = (TS_Color*)color->GetPointerFromInternalField(0);

	TS_Rectangle(x, y, w, h, c->toInt());

	return v8::Undefined();
}

void TS_Rectangle(int x, int y, int w, int h, unsigned int color) {
SDL_Rect rect = {(short int)x, (short int)y, (short unsigned int)w, (short unsigned int)h};
SDL_Rect locrect = {0, 0, (short unsigned int)w, (short unsigned int)h};

	SDL_Surface* surface = SDL_CreateRGBSurface(SDL_HWSURFACE|SDL_SRCALPHA, w, h, DEPTH, CHANNEL_MASKS);
	SDL_FillRect(surface, &locrect, color);
	SDL_BlitSurface(surface, NULL, screen, &rect);
    SDL_FreeSurface(surface);
}

v8::Handle<v8::Value> Line(const v8::Arguments& args)
{
    if(args.Length()<5){
        return v8::ThrowException(v8::String::New("TS_Rectangle Error: Called with fewer than 5 arguments!"));
	}
    CHECK_ARG_INT(0, "TS_Rectangle Error: Arg 0 is not a number.");
    CHECK_ARG_INT(1, "TS_Rectangle Error: Arg 1 is not a number.");
    CHECK_ARG_INT(2, "TS_Rectangle Error: Arg 2 is not a number.");
    CHECK_ARG_INT(3, "TS_Rectangle Error: Arg 3 is not a number.");
    CHECK_ARG_OBJ(4, "TS_Rectangle Error: Arg 4 is not an object.");
	int x1 = args[0]->v8::Value::IntegerValue();
	int y1 = args[1]->v8::Value::IntegerValue();
	int x2 = args[2]->v8::Value::IntegerValue();
	int y2 = args[3]->v8::Value::IntegerValue();

	v8::Local<v8::Object> color = v8::Local<v8::Object>::Cast(args[4]);

    TS_Color* c = (TS_Color*)color->GetPointerFromInternalField(0);

	TS_Line(x1, y1, x2, y2, c);

	return v8::Undefined();
}

void TS_Line(int ax1, int ay1, int ax2, int ay2, TS_Color *c){

    int color = c->toInt();

    int w = abs(ax1-ax2);
    int h = abs(ay1-ay2);

    SDL_Rect rect = {(short int)min(ax1, ax2), (short int)min(ay1, ay2), (short unsigned int)w, (short unsigned int)h};

    if(ax1-ax2<=1&&ax1-ax2>=-1){
    SDL_Surface *surface = SDL_CreateRGBSurface(SDL_SWSURFACE|SDL_SRCALPHA, 1, h, DEPTH, CHANNEL_MASKS);
    SDL_Rect allsurf = {0, 0, 1, (short unsigned int)h};
    SDL_FillRect(surface, &allsurf, color);

	SDL_BlitSurface(surface, NULL, screen, &rect);
    SDL_FreeSurface(surface);

    return;
    }
    else if(ay1-ay2<=1&&ay1-ay2>=-1){
    SDL_Surface *surface = SDL_CreateRGBSurface(SDL_SWSURFACE|SDL_SRCALPHA, w, 1, DEPTH, CHANNEL_MASKS);
    SDL_Rect allsurf = {0, 0, (short unsigned int)w, 1};
    SDL_FillRect(surface, &allsurf, color);

	SDL_BlitSurface(surface, NULL, screen, &rect);
    SDL_FreeSurface(surface);

    return;
    }

    int x1 = ax1-min(ax1, ax2);
    int y1 = ay1-min(ay1, ay2);
    int x2 = ax2-min(ax1, ax2);
    int y2 = ay2-min(ay1, ay2);

    SDL_Rect temprect = {(short int)x1, (short int)y1, 1, 1};

	SDL_Surface *surface = SDL_CreateRGBSurface(SDL_SWSURFACE|SDL_SRCALPHA, w+1, h+1, DEPTH, CHANNEL_MASKS);

    int sx;
    int sy;
    if (x1<x2){
        sx  = 1;
    }
    else {
        sx  = -1;
    }
    if (y1<y2){
        sy  = 1;
    }
    else{
        sy  = -1;
    }
    float err = (float)(w-h);

    float e2;
    while(true){
        temprect.x = x1;
        temprect.y = y1;

        SDL_FillRect(surface, &temprect, color);

        if ((x1==x2)&&(y1==y2)){break;}
        e2 = 2*err;

        if (e2>-h){

            err-= h;
            x1 += sx;

        }
        if (e2<w){

            err += w;
            y1  += sy;

        }
    }

	SDL_BlitSurface(surface, NULL, screen, &rect);
    SDL_FreeSurface(surface);
}

std::vector<TS_Point*> TS_GetLinePointsPoint(TS_Point *pa1, TS_Point *pa2){

    std::vector<TS_Point*> points;

    int w = abs(pa1->x-pa2->x);
    int h = abs(pa1->y-pa2->y);


    TS_Point *p1 = new TS_Point(pa1->x-min(pa1->x, pa2->x), pa1->y-min(pa1->y, pa2->y));
    TS_Point *p2 = new TS_Point(pa2->x-min(pa1->x, pa2->x), pa2->y-min(pa1->y, pa2->y));

    int sx;
    int sy;
    if (p1->x<p2->x){
        sx  = 1;
    }
    else {
        sx  = -1;
    }
    if (p1->y<p1->y){
        sy  = 1;
    }
    else{
        sy  = -1;
    }
    float err = (float)(w-h);

    float e2;

    TS_Point temppoint(p1->x, p1->y);

    while(true){
        temppoint.x = pa1->x;
        temppoint.y = pa1->y;

        points.push_back(new TS_Point(temppoint));
        if ((p1->x==p1->x)&&(p1->y==p1->y)){break;}
        e2 = 2*err;
        if (e2>-h){
            err-= h;
            p1->x += sx;
        }
        if (e2<w){
            err += w;
            p1->y  += sy;
        }
    }

    return points;
}

std::vector<TS_Point*> TS_GetLinePoints(TS_Point *pa1, TS_Point *pa2){

    std::vector<TS_Point*> points;

    int ax1 = pa1->x;
    int ay1 = pa1->y;
    int ax2 = pa2->x;
    int ay2 = pa2->y;

    int w = abs(pa1->x-pa2->x);
    int h = abs(pa1->y-pa2->y);

    int x1 = ax1;
    int y1 = ay1;
    int x2 = ax2;
    int y2 = ay2;

    int sx;
    int sy;
    if (x1<x2){
        sx  = 1;
    }
    else {
        sx  = -1;
    }
    if (y1<y2){
        sy  = 1;
    }
    else{
        sy  = -1;
    }
    float err = (float)(w-h);

    float e2;

    while(true){
        TS_Point *point = new TS_Point(x1, y1);
        points.push_back(point);
        if ((x1==x2)&&(y1==y2)){break;}
        e2 = 2*err;
        if (e2>-h){
            err-= h;
            x1 += sx;
        }
        if (e2<w){
            err += w;
            y1  += sy;
        }
    }

    return points;
}


void TS_OutlinedPolygon(std::vector<int> points, TS_Color *color){

    //verify we have enough points and an even number.
    if(points.size()%2!=0){
        return;
    }

    if(points.size()<4){
        return;
    }

    int size = (int)points.size();

    //the first line uses points that need wrap around.
    TS_Line(points[size-2], points[size-1], points[0], points[1], color);

    //cycle through the rest of the points.
    for(int i = 2; i<size-2; i+=2){
        TS_Line(points[i], points[i+1], points[i+3], points[i+4], color);
    }
    //
}


void TS_FilledPolygon(SDL_Surface *surface, std::vector<TS_Point*> points, TS_Color *c){

    TS_Point *minPoint = new TS_Point(points[0]->x, points[0]->y);
    TS_Point *maxPoint = new TS_Point(points[0]->x, points[0]->y);

    for(size_t i = 0; i<points.size(); i++){
        if(minPoint->x>points[i]->x){
            minPoint->x=points[i]->x;
        }
        if(minPoint->y>points[i]->y){
            minPoint->y=points[i]->y;
        }
        if(maxPoint->x<points[i]->x){
            maxPoint->x=points[i]->x;
        }
        if(maxPoint->y<points[i]->y){
            maxPoint->y=points[i]->y;
        }
    }

    int w = maxPoint->x-minPoint->x;
    int h = maxPoint->y-minPoint->y;

    std::vector<std::vector<TS_Point*> > pixels;

    std::vector<bool> crossedLinesMaster;

    for(size_t i = 0; i<=points.size(); i++){
        pixels.push_back(TS_GetLinePoints(points[i%(int)points.size()], points[(i+1)%(int)points.size()]));
        crossedLinesMaster.push_back(false);
    }

    int pxsize = pixels.size();

    SDL_Rect temprect = {0, 0, 1, 1};

    unsigned int color = c->toInt();

    for(int i = 0; i<pxsize; i++){
        for(size_t e = 0; e<pixels[i].size(); e++){
                temprect.x=pixels[i][e]->x;
                temprect.y=pixels[i][e]->y;
        }
    }

    int parity;

    std::vector<bool> crossedLines;

    for(int y = minPoint->y-1; y<h+minPoint->y; y++){
        //crossedLines.assign(crossedLines.size(), false);
        crossedLines = crossedLinesMaster;
        parity = 0;
        for(int x = minPoint->x-1; x<w+minPoint->x; x++){
            temprect.x = x;
            temprect.y = y;
            for(size_t i = 0; i<pixels.size(); i++){
                if(crossedLines[i]) {
                    continue;
                }

                //Skip the first pixel. It will always overlap another line's last pixel, and cause the parity to be double switched.
                for(size_t e = 1; e<pixels[i].size(); e++){
                    if(pixels[i][e]->x==x){
                        if(pixels[i][e]->y==y){
                            parity++;
                            crossedLines[i] = true;
                            break;
                        }
                    }
                }

            }
            //if(!parity) continue;

            if(!(parity%2)){
                SDL_FillRect(surface, &temprect, color);
            }

        }
    }

    for(size_t i = 0; i<pixels.size(); i++){
        for(size_t e = 0; e<pixels[i].size(); e++){
            delete pixels[i][e];
        }
        pixels[i].resize(0);
    }

    pixels.resize(0);
    crossedLinesMaster.resize(0);
    crossedLines.resize(0);

	delete minPoint;
	delete maxPoint;
}

v8::Handle<v8::Value> OutlinedPolygon(const v8::Arguments& args)
{
    if(args.Length()<2){
        return v8::ThrowException(v8::String::New("TS_OutlinedPolygon Error: Called with fewer than 2 arguments!"));
	}
    CHECK_ARG_ARRAY(0, "TS_Rectangle Error: Arg 0 is not an array.");
    CHECK_ARG_OBJ(1, "TS_Rectangle Error: Arg 1 is not an object.");

//	v8::Local<v8::Object> color = v8::Local<v8::Object>::Cast(args[1]);

	return v8::Undefined();
}

v8::Handle<v8::Value> TS_Test(const v8::Arguments& args)
{
    std::vector<TS_Point*> points;

    points.push_back(new TS_Point(100, 100));
    points.push_back(new TS_Point(200, 200));
    points.push_back(new TS_Point(230, 250));
    points.push_back(new TS_Point(100, 200));

    TS_Color c(255, 255, 255, 255);

    TS_FilledPolygon(screen, points,  &c);

    return v8::Undefined();
}


v8::Handle<v8::Value> GradientRectangle(const v8::Arguments& args)
{

    if(args.Length()<8){
        return v8::ThrowException(v8::String::New("TS_GradientRectangle Error: Called with fewer than 8 arguments!"));
	}
    CHECK_ARG_INT(0, "TS_GradientRectangle Error: Arg 0 is not a number.");
    CHECK_ARG_INT(1, "TS_GradientRectangle Error: Arg 1 is not a number.");
    CHECK_ARG_INT(2, "TS_GradientRectangle Error: Arg 2 is not a number.");
    CHECK_ARG_INT(3, "TS_GradientRectangle Error: Arg 3 is not a number.");
    CHECK_ARG_OBJ(4, "TS_GradientRectangle Error: Arg 4 is not an object.");
    CHECK_ARG_OBJ(5, "TS_GradientRectangle Error: Arg 5 is not an object.");
    CHECK_ARG_OBJ(6, "TS_GradientRectangle Error: Arg 6 is not an object.");
    CHECK_ARG_OBJ(7, "TS_GradientRectangle Error: Arg 7 is not an object.");


    int x = args[0]->v8::Value::IntegerValue();
	int y = args[1]->v8::Value::IntegerValue();
	int w = args[2]->v8::Value::IntegerValue();
	int h = args[3]->v8::Value::IntegerValue();

	v8::Local<v8::Object> color1 = v8::Local<v8::Object>::Cast(args[4]);
	v8::Local<v8::Object> color2 = v8::Local<v8::Object>::Cast(args[5]);
	v8::Local<v8::Object> color3 = v8::Local<v8::Object>::Cast(args[6]);
	v8::Local<v8::Object> color4 = v8::Local<v8::Object>::Cast(args[7]);

    TS_Color* c1 = (TS_Color*)color1->GetPointerFromInternalField(0);
    TS_Color* c2 = (TS_Color*)color2->GetPointerFromInternalField(0);
    TS_Color* c3 = (TS_Color*)color3->GetPointerFromInternalField(0);
    TS_Color* c4 = (TS_Color*)color4->GetPointerFromInternalField(0);

	TS_GradientRectangle(x, y, w, h, c1, c2, c3, c4);
	return v8::Undefined();
}

void TS_GradientRectangle(int x, int y, int w, int h, TS_Color *c1, TS_Color *c2, TS_Color *c3, TS_Color *c4){
    SDL_Rect rect       = {(short int)x, (short int)y, (short unsigned int)w, (short unsigned int)h};
    SDL_Rect temprect   = {0, 0, 1, 1};
    SDL_Surface* surface = SDL_CreateRGBSurface(SDL_SWSURFACE|SDL_SRCALPHA, w, h, DEPTH, CHANNEL_MASKS);

    TS_Color color(0, 0, 0, 0);

    int rDeltaTop = c2->red   - c1->red;
    int gDeltaTop = c2->green - c1->green;
    int bDeltaTop = c2->blue  - c1->blue;
    int aDeltaTop = c2->alpha - c1->alpha;

    int rDeltaLow = c3->red   - c4->red;
    int gDeltaLow = c3->green - c4->green;
    int bDeltaLow = c3->blue  - c4->blue;
    int aDeltaLow = c3->alpha - c4->alpha;

    color.red   = c1->red;
    color.green = c1->green;
    color.blue  = c1->blue;
    color.alpha = c1->alpha;

    int rLineDelta;
    int gLineDelta;
    int bLineDelta;
    int aLineDelta;

    TS_Color topColor = *c1;
    TS_Color lowColor = *c4;

    for(int tx = 0; tx<w; tx++){
        rLineDelta = lowColor.red   -topColor.red;
        gLineDelta = lowColor.green -topColor.green;
        bLineDelta = lowColor.blue  -topColor.blue;
        aLineDelta = lowColor.alpha -topColor.alpha;
        for(int ty = 0; ty<h; ty++){

            SDL_FillRect(surface, &temprect, color.toInt());
            color.red  =topColor.red  + (rLineDelta*ty/h);
            color.green=topColor.green+ (gLineDelta*ty/h);
            color.blue =topColor.blue + (bLineDelta*ty/h);
            color.alpha=topColor.alpha+ (aLineDelta*ty/h);
            temprect.y=ty;
        }

        topColor.red  =c1->red  + (rDeltaTop*tx/w);
        topColor.green=c1->green+ (gDeltaTop*tx/w);
        topColor.blue =c1->blue + (bDeltaTop*tx/w);
        topColor.alpha=c1->alpha+ (aDeltaTop*tx/w);

        lowColor.red  =c4->red  + (rDeltaLow*tx/w);
        lowColor.green=c4->green+ (gDeltaLow*tx/w);
        lowColor.blue =c4->blue + (bDeltaLow*tx/w);
        lowColor.alpha=c4->alpha+ (aDeltaLow*tx/w);

        temprect.x=tx;
    }

	SDL_BlitSurface(surface, NULL, screen, &rect);
	SDL_FreeSurface(surface);
}

void TS_FilledCircle(int x, int y, int rad, Uint8 r, Uint8 g, Uint8 b, Uint8 a) {
SDL_Rect rect = {(short int)(x-rad),(short int)(y-rad),(short unsigned int)(rad*2),(short unsigned int)(rad*2)};
unsigned int color = concatRGBA(r, g, b, a);
	SDL_Surface* surface;
	surface = SDL_CreateRGBSurface(SDL_HWSURFACE|SDL_SRCALPHA, rad*2, rad*2, DEPTH, CHANNEL_MASKS);
	for(int e = 0; e<rad; e++){
		for(int i = 0; i<rad; i++){
			if(sqrt(float(((rad-i)*(rad-i))+((rad-e)*(rad-e))))<rad){
			SDL_Rect locrect1 = {(short int)i,(short int)((2*rad)-e-1),(short unsigned int)((rad-i)*2),1};
			SDL_Rect locrect2 = {(short int)i,(short int)e,(short unsigned int)((rad-i)*2),1};
			SDL_FillRect(surface, &locrect1, color);
			SDL_FillRect(surface, &locrect2, color);
			i=rad;
			}

		}
	}
	SDL_BlitSurface(surface, NULL, screen, &rect);
	SDL_FreeSurface(surface);
}

void TS_OutlinedCircle(int x, int y, int rad, Uint8 r, Uint8 g, Uint8 b, Uint8 a) {
SDL_Rect rect = {(short int)(x-rad), (short int)(y-rad), (short unsigned int)(rad*2), (short unsigned int)(rad*2)};
unsigned int color = concatRGBA(r, g, b, a);
	SDL_Surface* surface;
	surface = SDL_CreateRGBSurface(SDL_HWSURFACE|SDL_SRCALPHA, rad*2, rad*2, DEPTH, CHANNEL_MASKS);
	for(int e = 0; e<rad; e++){
		for(int i = 0; i<=rad; i++){
			if(sqrt(double(((rad-i)*(rad-i))+((rad-e)*(rad-e))))<rad){

				SDL_Rect r1 = {(short int)i, (short int)e, 1, 1};
				SDL_Rect r2 = {(short int)i,(short int)((2*rad)-e-1), 1, 1};
				SDL_Rect r3 = {(short int)(i+(rad-i)*2), (short int)e, 1, 1};
				SDL_Rect r4 = {(short int)(i+(rad-i)*2), (short int)((2*rad)-e-1), 1, 1};
				SDL_FillRect(surface, &r1, color);
				SDL_FillRect(surface, &r2, color);
				SDL_FillRect(surface, &r3, color);
				SDL_FillRect(surface, &r4, color);
			i=rad;
			}

		}
	}
		for(int i = 0; i<=rad; i++){
			for(int e = 0; e<rad; e++){
			if(sqrt(double(((rad-i)*(rad-i))+((rad-e)*(rad-e))))<rad){

				SDL_Rect r1 = {(short int)i, (short int)e, 1, 1};
				SDL_Rect r2 = {(short int)i, (short int)((2*rad)-e-1), 1, 1};
				SDL_Rect r3 = {(short int)(i+(rad-i)*2), (short int)e, 1, 1};
				SDL_Rect r4 = {(short int)(i+(rad-i)*2), (short int)((2*rad)-e-1), 1, 1};
				SDL_FillRect(surface, &r1, color);
				SDL_FillRect(surface, &r2, color);
				SDL_FillRect(surface, &r3, color);
				SDL_FillRect(surface, &r4, color);
			e=rad;
			}

		}
	}
	SDL_BlitSurface(surface, NULL, screen, &rect);
	SDL_FreeSurface(surface);
}

void TS_FlipScreen()
{
	TendEvents();
    SDL_Flip(SDL_GetVideoSurface());
}




void TS_ShowSurface(SDL_Surface *image, int x, int y)
{
	SDL_Rect dest = {(short int)x, (short int)y, (short unsigned int)(image->w), (short unsigned int)(image->h)};
	SDL_BlitSurface(image, NULL, screen, &dest);
}

void TS_StretchShowSurface(SDL_Surface *image, int _x, int _y, int fh, int fv)
{
    //float   w = (float)(image->w*fh);
    //float   h = (float)(image->h*fv);
    int     wi = image->w*fh;
    int     hi = image->h*fv;

	SDL_Surface *surface = SDL_CreateRGBSurface(SDL_SWSURFACE|SDL_SRCALPHA, wi, wi, DEPTH, CHANNEL_MASKS);
    for(int x = 0; x<wi; x++){
        for(int y = 0; y<hi; y++){
            ((unsigned int*)surface->pixels)[(wi*y)+x]=((unsigned int*)image->pixels)[(int)(((y/fv)*image->w)+(x/fh))];
        }
    }
    TS_ShowSurface_Clear(surface, _x, _y);
}

v8::Handle<v8::Value> TS_surfaceZoomBlit(const v8::Arguments& args)
{
    if(args.Length()<3){
        return v8::ThrowException(v8::String::New("TS_surfaceZoomBlit Error: Called with fewer than 3 arguments!\n"));
    return v8::Undefined();
    }
    CHECK_ARG_INT(0, "TS_surfaceZoomBlit Error: Argument 0 is not a number.");
    CHECK_ARG_INT(1, "TS_surfaceZoomBlit Error: Argument 1 is not a number.");
    CHECK_ARG_INT(2, "TS_surfaceZoomBlit Error: Argument 2 is not a number.");

	v8::Local<v8::Object> self = args.Holder();
	v8::Local<v8::External> wrap = v8::Local<v8::External>::Cast(self->GetInternalField(0));
	void* ptr = wrap->Value();
	int factor = args[2]->v8::Value::IntegerValue();
	SDL_Surface* surface = static_cast<SDL_Surface*>(ptr);
	SDL_Rect dest = {(short int)(args[0]->v8::Value::NumberValue()), (short int)(args[1]->v8::Value::NumberValue()), (short unsigned int)(surface->w*factor), (short unsigned int)(surface->h*factor)};
	SDL_SoftStretch(surface, NULL, screen, &dest);
	return v8::Undefined();
}

v8::Handle<v8::Value> TS_surfaceStretchBlit(const v8::Arguments& args)
{
    if(args.Length()<4){
        return v8::ThrowException(v8::String::New("TS_surfaceZoomBlit Error: Called with fewer than 3 arguments!\n"));
    return v8::False();
    }
    CHECK_ARG_INT(0, "TS_surfaceStretchBlit Error: Argument 0 is not a number.");
    CHECK_ARG_INT(1, "TS_surfaceStretchBlit Error: Argument 1 is not a number.");
    CHECK_ARG_INT(2, "TS_surfaceStretchBlit Error: Argument 2 is not a number.");
    CHECK_ARG_INT(3, "TS_surfaceStretchBlit Error: Argument 3 is not a number.");

	v8::Local<v8::Object> self = args.Holder();
	v8::Local<v8::External> wrap = v8::Local<v8::External>::Cast(self->GetInternalField(0));
	void* ptr = wrap->Value();
	int xfactor = args[2]->v8::Value::IntegerValue();
	int yfactor = args[3]->v8::Value::IntegerValue();
	SDL_Surface* surface = static_cast<SDL_Surface*>(ptr);
	//SDL_Rect dest = {args[0]->v8::Value::NumberValue(), args[1]->v8::Value::NumberValue(), xfactor, yfactor};
	//SDL_SoftStretch(surface, NULL, screen, &dest);
    TS_StretchShowSurface(surface, args[0]->v8::Value::IntegerValue(), args[1]->v8::Value::IntegerValue(), surface->w*xfactor, surface->h*yfactor);
	return v8::Undefined();
}

v8::Handle<v8::Value> TS_surfaceZoomBlitSurface(const v8::Arguments& args)
{
    if(args.Length()<3){
        return v8::ThrowException(v8::String::New("TS_surfaceZoomBlitSurface Error: Called with fewer than 3 arguments!\n"));
    return v8::Undefined();
    }
    CHECK_ARG_INT(0, "TS_surfaceZoomBlitSurface Error: Argument 0 is not a number.");
    CHECK_ARG_INT(1, "TS_surfaceZoomBlitSurface Error: Argument 1 is not a number.");
    CHECK_ARG_OBJ(2, "TS_surfaceZoomBlitSurface Error: Argument 2 is not an object.");

	v8::Local<v8::Object> self = args.Holder();
	v8::Local<v8::External> wrap = v8::Local<v8::External>::Cast(self->GetInternalField(0));
	void* ptr = wrap->Value();
	int factor = args[2]->v8::Value::IntegerValue();
	SDL_Surface* surface = static_cast<SDL_Surface*>(ptr);
	//SDL_Rect dest = {args[0]->v8::Value::NumberValue(), args[1]->v8::Value::NumberValue(), surface->w*factor, surface->h*factor};
	//SDL_SoftStretch(surface, NULL, screen, &dest);
	TS_StretchShowSurface(surface, args[0]->v8::Value::IntegerValue(), args[1]->v8::Value::IntegerValue(), surface->w*factor, surface->h*factor);
	return v8::Undefined();
}

//A secret SDL function:
//SDL_SoftStretch(image, &src, screen, &dest);

v8::Handle<v8::Value> TS_surfaceBlit(const v8::Arguments& args)
{
    if(args.Length()<2){
	return v8::ThrowException(v8::String::New("TS_surface Error: Called with fewer than 2 arguments!\n"));
    }
    CHECK_ARG_INT(0, "TS_surfaceBlit Error: Argument 0 is not a number.");
    CHECK_ARG_INT(1, "TS_surfaceBlit Error: Argument 1 is not a number.");
	v8::Local<v8::Object> self = args.Holder();
	v8::Local<v8::External> wrap = v8::Local<v8::External>::Cast(self->GetInternalField(0));
	void* ptr = wrap->Value();
	TS_ShowSurface(static_cast<SDL_Surface*>(ptr), args[0]->v8::Value::NumberValue(), args[1]->v8::Value::NumberValue());
	return v8::Undefined();
}

v8::Handle<v8::Value> TS_surfaceBlitSurface(const v8::Arguments& args)
{
	if(args.Length()<3){
	return v8::ThrowException(v8::String::New("TS_surfaceBlitSurface Error: Called with fewer than 3 arguments!\n"));
	}

    CHECK_ARG_INT(0, "TS_surfaceBlit Error: Argument 0 is not a number.");
    CHECK_ARG_INT(1, "TS_surfaceBlit Error: Argument 1 is not a number.");

    v8::Local<v8::Object> self = args.Holder();
	v8::Local<v8::External> wrap = v8::Local<v8::External>::Cast(self->GetInternalField(0));
	void* ptr = wrap->Value();
	int x = args[1]->v8::Value::IntegerValue();
	int y = args[2]->v8::Value::IntegerValue();

	v8::Handle<v8::Object> surface = v8::Handle<v8::Object>::Cast(args[0]);
	v8::Local<v8::External> wraparg0 = v8::Local<v8::External>::Cast(surface->GetInternalField(0));
	void *ptrarg = wraparg0->Value();
	int w = surface->Get(v8::String::New("width"))->v8::Value::IntegerValue();
	int h = surface->Get(v8::String::New("height"))->v8::Value::IntegerValue();
	SDL_Rect srcrect = {0, 0, (short unsigned int)w, (short unsigned int)h};
	SDL_Rect dstrect = {(short int)x, (short int)y, (short unsigned int)w, (short unsigned int)h};
	SDL_BlitSurface( static_cast<SDL_Surface*>(ptrarg),&srcrect,static_cast<SDL_Surface*>(ptr),  &dstrect);
	return v8::Undefined();
}

v8::Handle<v8::Value> TS_surfaceSetPixel(const v8::Arguments& args)
{
	if(args.Length()<3){
	return v8::ThrowException(v8::String::New("TS_surfaceSetPixel Error: Called with fewer than 3 arguments!\n"));
	}

    CHECK_ARG_INT(0, "TS_surfaceSetPixel Error: Argument 0 is not a number.");
    CHECK_ARG_INT(1, "TS_surfaceSetPixel Error: Argument 1 is not a number.");
    CHECK_ARG_OBJ(2, "TS_surfaceSetPixel Error: Argument 2 is not an object.");

	int x = args[0]->v8::Value::IntegerValue();
	int y = args[1]->v8::Value::IntegerValue();

	v8::Handle<v8::Object> color = v8::Handle<v8::Object>::Cast(args[2]);

    TS_Color* c = (TS_Color*)color->GetPointerFromInternalField(0);
	unsigned int concatColor = c->toInt();

	SDL_Rect rect = {(short int)x, (short int)y, 1, 1};

	v8::Local<v8::Object> self = args.Holder();
	v8::Local<v8::External> wrap = v8::Local<v8::External>::Cast(self->GetInternalField(0));
	SDL_FillRect(static_cast<SDL_Surface*>(wrap->Value()), &rect, concatColor);
	return v8::Undefined();
}

v8::Handle<v8::Value> TS_surfaceGetPixel(const v8::Arguments& args);

v8::Handle<v8::Value> TS_surfaceStretchBlit(const v8::Arguments& args);

void TS_SurfaceFinalizer(v8::Persistent<v8::Value> object, void* parameter);

v8::Handle<v8::Value> TS_CreateSurface(const v8::Arguments& args)
{
    v8::HandleScope loadsurfacescope;
    v8::Handle<v8::Value> external;
    SDL_Surface *surface = NULL;
    if (args[0]->IsExternal()) {
        external = v8::Local<v8::External>::Cast(args[0]);
    }
    else {
        if(args[0]->IsString()){//Load Image from file
            v8::String::AsciiValue str(args[0]);
            const char *file = *str;

            surface = IMG_Load(string(TS_dirs.image).append(file).c_str());
            if(!surface) {
                printf("TS_CreateSurface: %s\n", IMG_GetError());
                return v8::ThrowException(v8::String::New(string("Could not open surface %s").append(file).c_str()));
            }
        }
        else if(args[0]->IsNumber()&&args[1]->IsNumber()){

            int w = args[0]->IntegerValue();
            int h = args[1]->IntegerValue();

            v8::Handle<v8::Object> color = v8::Handle<v8::Object>::Cast(args[2]);
            TS_Color* c = (TS_Color*)color->GetPointerFromInternalField(0);
            surface = TS_SurfaceFromSpec(w, h, c->toInt(), false);

        }
        else{
            return v8::ThrowException(v8::String::New("TS_CreateSurface error: Bad args. Must be string (filename) or number, number, Color_object"));
        }
        external = v8::External::New(surface);

    }
  	SurfaceInsttempl->SetInternalFieldCount(1);
	v8::Handle<v8::Function> Surfacector = Surfacetempl->GetFunction();

	v8::Local<v8::Object> Surfaceobj = Surfacector->NewInstance();

    v8::Persistent<v8::Object> Psurfaceobj = v8::Persistent<v8::Object>::New(Surfaceobj);
    Surfaceobj.Clear();
    Surfacector.Clear();

    Psurfaceobj.MakeWeak(surface, TS_SurfaceFinalizer);
	Psurfaceobj->SetInternalField(0, external);
    external.Clear();

    return loadsurfacescope.Close(Psurfaceobj);
}


v8::Handle<v8::Value> TS_CreateImage(const v8::Arguments& args)
{
    v8::HandleScope loadsurfacescope;
    v8::Handle<v8::Value> external;
	SDL_Surface *surface = NULL;
  //v8::Handle<v8::Object> wrapper;

    if(args.Length()<1){
        return v8::ThrowException(v8::String::New("TS_CreateImage Error: Called with no arguments."));
    }

    if (args[0]->IsExternal()) {
        external = v8::Local<v8::External>::Cast(args[0]);
    } else {
        if(args[0]->IsString()){//Load Image from file
            v8::String::AsciiValue str(args[0]);
            const char *file = *str;

            surface = IMG_Load(string(TS_dirs.image).append(file).c_str());
            if(!surface) {
                printf("TS_CreateImage: %s\n", IMG_GetError());
                return v8::ThrowException(v8::String::New(string("Could not open surface %s").append(file).c_str()));
            }
            //
        }
        else if(args[0]->IsNumber()&&args[1]->IsNumber()){

            int w = args[0]->IntegerValue();
            int h = args[1]->IntegerValue();

            v8::Handle<v8::Object> color = v8::Handle<v8::Object>::Cast(args[2]);
            TS_Color* c = (TS_Color*)color->GetPointerFromInternalField(0);
            surface = TS_SurfaceFromSpec(w, h, c->toInt(), true);

        }
        else{
            return v8::ThrowException(v8::String::New("TS_CreateImage error: Bad args. Must be string (filename) or number, number, Color_object"));
        }
        external = v8::External::New(surface);

    }

  	SurfaceInsttempl->SetInternalFieldCount(1);
	v8::Handle<v8::Function> Surfacector = Surfacetempl->GetFunction();

	v8::Local<v8::Object> Surfaceobj = Surfacector->NewInstance();

    v8::Persistent<v8::Object> Psurfaceobj = v8::Persistent<v8::Object>::New(Surfaceobj);
    Psurfaceobj.MakeWeak(surface, TS_SurfaceFinalizer);
	Psurfaceobj->SetInternalField(0, external);

    return loadsurfacescope.Close(Psurfaceobj);
}


v8::Handle<v8::Value> TS_GetSystemArrow(const v8::Arguments& args)
{
    v8::HandleScope loadsurfacescope;
    v8::Handle<v8::Value> external;
	SDL_Surface *surface = IMG_Load(string(TS_dirs.system).append(TS_conf.systemarrow).c_str());

    external = v8::External::New(surface);

  	SurfaceInsttempl->SetInternalFieldCount(1);
	v8::Handle<v8::Function> Surfacector = Surfacetempl->GetFunction();

	v8::Local<v8::Object> Surfaceobj = Surfacector->NewInstance();

    v8::Persistent<v8::Object> Psurfaceobj = v8::Persistent<v8::Object>::New(Surfaceobj);

    Psurfaceobj.MakeWeak(surface, TS_SurfaceFinalizer);
	Psurfaceobj->SetInternalField(0, external);

    return loadsurfacescope.Close(Psurfaceobj);
}


void TS_SurfaceFinalizer(v8::Persistent<v8::Value> object, void* parameter) {
    SDL_Surface* surface = (SDL_Surface*)parameter;
    SDL_FreeSurface(surface);
    object.Dispose();
    object.Clear();
    object.IsNearDeath();
}



TS_Color::TS_Color(Uint8 r, Uint8 g, Uint8 b, Uint8 a){
    red		= r;
    green	= g;
    blue	= b;
    alpha	= a;
}

TS_Color::TS_Color(void){
    red     = 0;
    green   = 0;
    blue    = 0;
    alpha   = 0;
}

unsigned int TS_Color::toInt(){
unsigned int c = concatRGBA(red, green, blue, alpha);
return c;
}

void TS_ColorFinalizer(v8::Persistent<v8::Value> object, void* parameter) {
    TS_Color* color = (TS_Color*)parameter;
    delete color;
    object.Dispose();
    object.Clear();
}

TS_Color::~TS_Color(){

}

v8::Handle<v8::Value> GetWidth(v8::Local<v8::String> property, const v8::AccessorInfo &info) {
    v8::Local<v8::Object> self = info.Holder();
    v8::Local<v8::External> wrap = v8::Local<v8::External>::Cast(self->GetInternalField(0));
    void* ptr = wrap->Value();
    int value = static_cast<SDL_Surface*>(ptr)->w;
    return v8::Integer::New(value);
  }

  void SetWidth(v8::Local<v8::String> property, v8::Local<v8::Value> value, const v8::AccessorInfo& info) {
    v8::Local<v8::Object> self = info.Holder();
    v8::Local<v8::External> wrap = v8::Local<v8::External>::Cast(self->GetInternalField(0));
    void* ptr = wrap->Value();
    static_cast<SDL_Surface*>(ptr)->w = value->Uint32Value();
  }

  v8::Handle<v8::Value> GetHeight(v8::Local<v8::String> property, const v8::AccessorInfo &info) {
    v8::Local<v8::Object> self = info.Holder();
    v8::Local<v8::External> wrap = v8::Local<v8::External>::Cast(self->GetInternalField(0));
    void* ptr = wrap->Value();
    int value = static_cast<SDL_Surface*>(ptr)->h;
    return v8::Integer::New(value);
  }

  void SetHeight(v8::Local<v8::String> property, v8::Local<v8::Value> value, const v8::AccessorInfo& info) {
    v8::Local<v8::Object> self = info.Holder();
    v8::Local<v8::External> wrap = v8::Local<v8::External>::Cast(self->GetInternalField(0));
    void* ptr = wrap->Value();
	static_cast<SDL_Surface*>(ptr)->h = value->Uint32Value();
  }

v8::Handle<v8::Value> GetRed(v8::Local<v8::String> property, const v8::AccessorInfo &info) {
    v8::Local<v8::Object> self = info.Holder();
    v8::Local<v8::External> wrap = v8::Local<v8::External>::Cast(self->GetInternalField(0));
    void* ptr = wrap->Value();
    int value = static_cast<TS_Color*>(ptr)->red;
    return v8::Integer::New(value);
  }

  void SetRed(v8::Local<v8::String> property, v8::Local<v8::Value> value, const v8::AccessorInfo& info) {
    v8::Local<v8::Object> self = info.Holder();
    v8::Local<v8::External> wrap = v8::Local<v8::External>::Cast(self->GetInternalField(0));
    void* ptr = wrap->Value();
    static_cast<TS_Color*>(ptr)->red = value->Int32Value();
  }

v8::Handle<v8::Value> GetGreen(v8::Local<v8::String> property,
                          const v8::AccessorInfo &info) {
    v8::Local<v8::Object> self = info.Holder();
    v8::Local<v8::External> wrap = v8::Local<v8::External>::Cast(self->GetInternalField(0));
    void* ptr = wrap->Value();
    int value = static_cast<TS_Color*>(ptr)->green;
    return v8::Integer::New(value);
  }

  void SetGreen(v8::Local<v8::String> property, v8::Local<v8::Value> value,
                 const v8::AccessorInfo& info) {
    v8::Local<v8::Object> self = info.Holder();
    v8::Local<v8::External> wrap = v8::Local<v8::External>::Cast(self->GetInternalField(0));
    void* ptr = wrap->Value();
    static_cast<TS_Color*>(ptr)->green = value->Int32Value();
  }


v8::Handle<v8::Value> GetBlue(v8::Local<v8::String> property,
                          const v8::AccessorInfo &info) {
    v8::Local<v8::Object> self = info.Holder();
    v8::Local<v8::External> wrap = v8::Local<v8::External>::Cast(self->GetInternalField(0));
    void* ptr = wrap->Value();
    int value = static_cast<TS_Color*>(ptr)->blue;
    return v8::Integer::New(value);
  }

  void SetBlue(v8::Local<v8::String> property, v8::Local<v8::Value> value,
                 const v8::AccessorInfo& info) {
    v8::Local<v8::Object> self = info.Holder();
    v8::Local<v8::External> wrap = v8::Local<v8::External>::Cast(self->GetInternalField(0));
    void* ptr = wrap->Value();
    static_cast<TS_Color*>(ptr)->blue = value->Int32Value();
  }

v8::Handle<v8::Value> GetAlpha(v8::Local<v8::String> property,
                          const v8::AccessorInfo &info) {
    v8::Local<v8::Object> self = info.Holder();
    v8::Local<v8::External> wrap = v8::Local<v8::External>::Cast(self->GetInternalField(0));
    void* ptr = wrap->Value();
    int value = static_cast<TS_Color*>(ptr)->alpha;
    return v8::Integer::New(value);
  }

  void SetAlpha(v8::Local<v8::String> property, v8::Local<v8::Value> value,
                 const v8::AccessorInfo& info) {
    v8::Local<v8::Object> self = info.Holder();
    v8::Local<v8::External> wrap = v8::Local<v8::External>::Cast(self->GetInternalField(0));
    void* ptr = wrap->Value();
    static_cast<TS_Color*>(ptr)->alpha = value->Int32Value();
  }




v8::Handle<v8::Value> CreateColor(const v8::Arguments& args) {
  v8::HandleScope loadcolorscope;
   v8::Handle<v8::Value> external;

  TS_Color *color = new TS_Color(255, 255, 255, 255);
    if(args.Length()<3){
        return v8::ThrowException(v8::String::New("CreateColor Error: Called with fewer than 3 arguments!"));
    }
    if (args[0]->IsExternal()) {
        external = v8::Local<v8::External>::Cast(args[0]);
    } else {


    CHECK_ARG_INT(0, "CreateColor Error: Could not parse arg 0 to number!");
    CHECK_ARG_INT(1, "CreateColor Error: Could not parse arg 1 to number!");
    CHECK_ARG_INT(2, "CreateColor Error: Could not parse arg 2 to number!");

    int r = args[0]->Int32Value();
    int g = args[1]->Int32Value();
    int b = args[2]->Int32Value();
    int a = 255;
    if(args.Length()>3){
        CHECK_ARG_INT(3, "CreateColor Error: Could not parse arg 3 to number!");
        a = args[3]->Int32Value();
    }

    color->red      = r;
    color->green    = g;
    color->blue     = b;
    color->alpha    = a;
    external = v8::External::New(color);
  }

  	ColorInsttempl->SetInternalFieldCount(1);
	v8::Handle<v8::Function> colorctor = Colortempl->GetFunction();
	v8::Local<v8::Object> colorobj = colorctor->NewInstance();

    //colorobj->SetInternalField(0, external);
	v8::Persistent<v8::Object> Pcolorobj = v8::Persistent<v8::Object>::New(colorobj);//colorctor->NewInstance();

    colorobj.Clear();
    colorctor.Clear();

    Pcolorobj.MakeWeak(color, TS_ColorFinalizer);
    Pcolorobj->SetInternalField(0, external);

    external.Clear();
	return loadcolorscope.Close(Pcolorobj);
}



v8::Handle<v8::Value> TS_surfaceGetPixel(const v8::Arguments& args){
	if(args.Length()<2){
		printf("TS_surfaceGetPixel Error: Called with fewer than 2 arguments!");
		return v8::False();
		}

	v8::HandleScope loadcolorscope;
	v8::Handle<v8::Value> external;
	int x = args[0]->v8::Value::IntegerValue();
	int y = args[1]->v8::Value::IntegerValue();

	v8::Local<v8::Object> self = args.Holder();
	v8::Local<v8::External> wrap = v8::Local<v8::External>::Cast(self->GetInternalField(0));
	void *ptr = wrap->Value();
	//SDL_Surface *surf = static_cast<SDL_Surface*>(ptr);
	//unsigned int color = ((unsigned int*)surf->pixels)[y*(surf->pitch/sizeof(unsigned int)) + x];
	unsigned int color = ((unsigned int*)static_cast<SDL_Surface*>(ptr)->pixels)[y*(static_cast<SDL_Surface*>(ptr)->pitch/sizeof(unsigned int)) + x];

	Uint8 b = (color<<8)>>24;
	Uint8 g = (color<<16)>>24;
	Uint8 r = (color<<24)>>24;
	Uint8 a = (color<<0)>>24;
    TS_Color *retcolor = new TS_Color(r, g, b, a);
    external = v8::External::New(retcolor);

  	ColorInsttempl->SetInternalFieldCount(1);
	v8::Handle<v8::Function> colorctor = Colortempl->GetFunction();
		//get class instance
	v8::Local<v8::Object> colorobj = colorctor->NewInstance();
		//Bridge the JS class seeds by setting the fields
	colorobj->SetInternalField(0, external);

	return loadcolorscope.Close(colorobj);

}
/*
void TS_TexturedQuad(SDL_Surface *dest, SDL_Surface *texture, TS_Point Pul, TS_Point Pur, TS_Point Pbl, TS_Point Pbr){

    TS_Point UL = Pul;
    TS_Point UR = Pur;
    TS_Point BL = Pbl;
    TS_Point BR = Pbr;

    hdir = 1;
    vdir = 1;

    if (sqrtf(abs(Pul.x - Pbl.x)+abs(Pul.y - Pbl.y))>sqrtf(abs(Pur.x - Pbr.x)+abs(Pur.y - Pbr.y))){
        UL = Pur;
        UR = Pul;
        BL = Pbr;
        BR = Pbl;
        hdir = -1;
    }

    if (sqrtf(abs(Pur.x - Pul.x)+abs(Pur.y - Pul.y))>sqrtf(abs(Pbl.x - Pbr.x)+abs(Pbl.y - Pbr.y))){
        TS_Point temp;

        UL =
        UR =
        BL =
        BR =
        hdir = -1;
    }

    //With the

}
*/
