#ifndef GRAPHIC_HEAD
#define GRAPHIC_HEAD
#include "SDL_V8.h"
#include <vector>

void SurfaceInit();
void SurfaceClose();
void ImageInit();
void ImageClose();
void ColorInit();
void ColorClose();

extern v8::Persistent<v8::FunctionTemplate> Surfacetempl;
extern v8::Persistent<v8::ObjectTemplate> SurfaceInsttempl;

extern v8::Persistent<v8::FunctionTemplate> Imagetempl;
extern v8::Persistent<v8::ObjectTemplate> ImageInsttempl;

extern v8::Persistent<v8::FunctionTemplate> Colortempl;
extern v8::Persistent<v8::ObjectTemplate> ColorInsttempl;

inline int concatRGBA(int r, int g, int b, int a);

void TS_FlipScreen();

void TS_ShowSurface(SDL_Surface *image, int x, int y);
void TS_ShowSurface_Clear(SDL_Surface *image, int x, int y);

class TS_Color{
public:
	TS_Color(Uint8, Uint8, Uint8, Uint8);
	TS_Color(void);
	~TS_Color();
	unsigned int toInt();
		Uint8 red, green, blue, alpha;
};

typedef struct TS_Point {
	int x;
	int y;
	TS_Point operator=(const SDL_Rect &c2);
    TS_Point(int, int);
    ~TS_Point();
} TS_Point;

typedef struct TS_Segment {
	int x1;
	int y1;
	int x2;
	int y2;
    TS_Segment(int, int, int, int);
    TS_Segment();
    ~TS_Segment();
} TS_Segment;




void TS_ColorFinalizer(v8::Persistent<v8::Value> object, void* parameter);

void TS_GradientRectangle(int x, int y, int w, int h, TS_Color *c1, TS_Color *c2, TS_Color *c3, TS_Color *c4);
void TS_Rectangle(int x, int y, int w, int h, unsigned int color);
void TS_drawAlphaRect(int x, int y, int w, int h, int c, int a, SDL_Rect rect);
void TS_Rectangle(int x, int y, int w, int h, Uint8 r, Uint8 g, Uint8 b, Uint8 a);
void TS_FilledCircle(int x, int y, int rad, Uint8 r, Uint8 g, Uint8 b, Uint8 a);
void TS_OutlinedCircle(int x, int y, int rad, Uint8 r, Uint8 g, Uint8 b, Uint8 a);
void TS_Line(int x1, int y1, int x2, int y2, TS_Color *);

//accessors
v8::Handle<v8::Value> Point(const v8::Arguments& args);
v8::Handle<v8::Value> Rectangle(const v8::Arguments& args);
v8::Handle<v8::Value> GradientRectangle(const v8::Arguments& args);
v8::Handle<v8::Value> FilledCircle(const v8::Arguments& args);
v8::Handle<v8::Value> OutlinedCircle(const v8::Arguments& args);
v8::Handle<v8::Value> TS_surfaceBlit(const v8::Arguments& args);
v8::Handle<v8::Value> TS_surfaceBlitSurface(const v8::Arguments& args);
v8::Handle<v8::Value> TS_surfaceSetPixel(const v8::Arguments& args);
v8::Handle<v8::Value> TS_CreateSurface(const v8::Arguments& args);
v8::Handle<v8::Value> TS_CreateImage(const v8::Arguments& args);
v8::Handle<v8::Value> CreateColor(const v8::Arguments& args);
v8::Handle<v8::Value> TS_surfaceGetPixel(const v8::Arguments& args);
v8::Handle<v8::Value> Line(const v8::Arguments& args);

//surfaces
v8::Handle<v8::Value> GetWidth(v8::Local<v8::String> property, const v8::AccessorInfo &info);
void SetWidth(v8::Local<v8::String> property, v8::Local<v8::Value> value, const v8::AccessorInfo& info);
v8::Handle<v8::Value> GetHeight(v8::Local<v8::String> property, const v8::AccessorInfo &info);
void SetHeight(v8::Local<v8::String> property, v8::Local<v8::Value> value, const v8::AccessorInfo& info);

//colors
v8::Handle<v8::Value> GetRed(v8::Local<v8::String> property, const v8::AccessorInfo &info);
void SetRed(v8::Local<v8::String> property, v8::Local<v8::Value> value, const v8::AccessorInfo& info);
v8::Handle<v8::Value> GetGreen(v8::Local<v8::String> property, const v8::AccessorInfo &info);
void SetGreen(v8::Local<v8::String> property, v8::Local<v8::Value> value, const v8::AccessorInfo& info);
v8::Handle<v8::Value> GetBlue(v8::Local<v8::String> property, const v8::AccessorInfo &info);
void SetBlue(v8::Local<v8::String> property, v8::Local<v8::Value> value, const v8::AccessorInfo& info);
v8::Handle<v8::Value> GetAlpha(v8::Local<v8::String> property, const v8::AccessorInfo &info);
void SetAlpha(v8::Local<v8::String> property, v8::Local<v8::Value> value, const v8::AccessorInfo& info);

//system images
v8::Handle<v8::Value> TS_GetSystemArrow(const v8::Arguments& args);

v8::Handle<v8::Value> TS_Test(const v8::Arguments& args);

#define TS_StretchShowSurface_Clear(image, _x, _y, fh, fv){ \
TS_StretchShowSurface(image, _x, _y, fh, fv);\
SDL_FreeSurface(image);\
}

#define TS_ShowSurface_Clear(image, _x, _y){ \
TS_ShowSurface(image, _x, _y);\
SDL_FreeSurface(image);\
}

#endif
