#include "SDL_V8.h"
#include <ctype.h>

	v8::Persistent<v8::FunctionTemplate> KeyConsttempl;//   = *v8::FunctionTemplate::New();
	v8::Persistent<v8::ObjectTemplate> KeyConstInsttempl;// = *KeyConsttempl->InstanceTemplate();

void KeyInit(){
    SDL_EnableUNICODE(SDL_ENABLE);
    KeyConsttempl   = *v8::FunctionTemplate::New();
    KeyConstInsttempl = *KeyConsttempl->InstanceTemplate();
}

void KeyClose(){
    KeyConsttempl.Dispose();
    KeyConstInsttempl.Dispose();
}

v8::Handle<v8::Value> GetKeyString(const v8::Arguments& args){
	if(args.Length()<1){
        return v8::ThrowException(v8::String::New("GetKeyString Error: Called with no arguments"));
	}
	CHECK_ARG_INT(0, "GetKeyString Error: Argument 0 is not a number.");

    bool shift = false;

	if(args.Length()>1){
        CHECK_ARG_BOOL(1, "GetKeyString Error: Argument 1 is not a boolean.");
        if(args[1]->v8::Value::BooleanValue()){
            shift = true;
        }
	}

    unsigned int keynum = args[0]->v8::Value::Int32Value();

    if(keynum>256){
        return v8::String::New("");
    }

    if(shift){
        keynum = toupper(keynum);
    }

    std::string key(1, keynum);

    return v8::String::New(key.c_str());
}

v8::Handle<v8::Value> GetKey(const v8::Arguments& args)
{
    TendEvents();
	//SDL_PollEvent( &event );
	while(!(event.type==SDL_KEYDOWN))
	{
			TendEvents();
			//SDL_PollEvent(&event);


	}
	int key = event.key.keysym.sym;
	if(!key) {
        printf("GetKey Error: You are probably using an unsupported keyboard layout\nSDLKey Error: %s\n", SDL_GetError());
    }
	return v8::Number::New(key);
}

v8::Handle<v8::Value> AreKeysLeft(const v8::Arguments& args){

    	TendEvents();
	if(!(event.type==SDL_KEYDOWN))
	{
			SDL_PeepEvents(&event, 1, SDL_ADDEVENT, SDL_ALLEVENTS);
			return v8::Boolean::New(false);
	}
    else{
    		SDL_PeepEvents(&event, 1, SDL_ADDEVENT, SDL_ALLEVENTS);
			return v8::Boolean::New(true);
    }
}

v8::Handle<v8::Value> IsKeyPressed(const v8::Arguments& args)
{
	if(args.Length()<1){return v8::Undefined();}
	SDL_PumpEvents();
	int key = args[0]->Int32Value();
	Uint8 *keystate = SDL_GetKeyState(NULL);
	if(keystate[key]){return v8::True();}
	else {return v8::False();}
}

v8::Handle<v8::Value> IsAnyKeyPressed(const v8::Arguments& args)
{
	SDL_PumpEvents();
	int numkeys = 343;
	int *keys = &numkeys;
	Uint8 *keystate = SDL_GetKeyState(keys);
	for(int i = 0; i<numkeys; i++){
		if(i==SDLK_SCROLLOCK||i==SDLK_NUMLOCK||i==SDLK_CAPSLOCK){
			continue;
		}
		if(keystate[i]){
        return v8::True();
        }
	}
    return v8::False();
}

void TS_SetKeyConstants(v8::Persistent<v8::Context> context){

	context->Global()->Set(v8::String::New("KEY_ENTER"),  v8::Number::New(SDLK_RETURN));

	context->Global()->Set(v8::String::New("KEY_A"),  v8::Number::New(SDLK_a));

	context->Global()->Set(v8::String::New("KEY_B"),  v8::Number::New(SDLK_b));

	context->Global()->Set(v8::String::New("KEY_C"),  v8::Number::New(SDLK_c));

	context->Global()->Set(v8::String::New("KEY_D"),  v8::Number::New(SDLK_d));

	context->Global()->Set(v8::String::New("KEY_E"),  v8::Number::New(SDLK_e));

	context->Global()->Set(v8::String::New("KEY_F"),  v8::Number::New(SDLK_f));

	context->Global()->Set(v8::String::New("KEY_G"),  v8::Number::New(SDLK_g));

	context->Global()->Set(v8::String::New("KEY_H"),  v8::Number::New(SDLK_h));

	context->Global()->Set(v8::String::New("KEY_I"),  v8::Number::New(SDLK_i));

	context->Global()->Set(v8::String::New("KEY_J"),  v8::Number::New(SDLK_j));

	context->Global()->Set(v8::String::New("KEY_K"),  v8::Number::New(SDLK_k));

	context->Global()->Set(v8::String::New("KEY_L"),  v8::Number::New(SDLK_l));

	context->Global()->Set(v8::String::New("KEY_M"),  v8::Number::New(SDLK_m));

	context->Global()->Set(v8::String::New("KEY_N"),  v8::Number::New(SDLK_n));

	context->Global()->Set(v8::String::New("KEY_O"),  v8::Number::New(SDLK_o));

	context->Global()->Set(v8::String::New("KEY_P"),  v8::Number::New(SDLK_p));

	context->Global()->Set(v8::String::New("KEY_Q"),  v8::Number::New(SDLK_q));

	context->Global()->Set(v8::String::New("KEY_R"),  v8::Number::New(SDLK_r));

	context->Global()->Set(v8::String::New("KEY_S"),  v8::Number::New(SDLK_s));

	context->Global()->Set(v8::String::New("KEY_T"),  v8::Number::New(SDLK_t));

	context->Global()->Set(v8::String::New("KEY_U"),  v8::Number::New(SDLK_u));

	context->Global()->Set(v8::String::New("KEY_V"),  v8::Number::New(SDLK_v));

	context->Global()->Set(v8::String::New("KEY_W"),  v8::Number::New(SDLK_w));

	context->Global()->Set(v8::String::New("KEY_X"),  v8::Number::New(SDLK_x));

	context->Global()->Set(v8::String::New("KEY_Y"),  v8::Number::New(SDLK_y));

	context->Global()->Set(v8::String::New("KEY_Z"),  v8::Number::New(SDLK_z));

	context->Global()->Set(v8::String::New("KEY_TAB"),  v8::Number::New(SDLK_TAB));

	context->Global()->Set(v8::String::New("KEY_ESCAPE"),  v8::Number::New(SDLK_ESCAPE));

	context->Global()->Set(v8::String::New("KEY_F1"),  v8::Number::New(SDLK_F1));

	context->Global()->Set(v8::String::New("KEY_F2"),  v8::Number::New(SDLK_F2));

	context->Global()->Set(v8::String::New("KEY_F3"),  v8::Number::New(SDLK_F3));

	context->Global()->Set(v8::String::New("KEY_F4"),  v8::Number::New(SDLK_F4));

	context->Global()->Set(v8::String::New("KEY_F5"),  v8::Number::New(SDLK_F5));

	context->Global()->Set(v8::String::New("KEY_F6"),  v8::Number::New(SDLK_F6));

	context->Global()->Set(v8::String::New("KEY_F7"),  v8::Number::New(SDLK_F7));

	context->Global()->Set(v8::String::New("KEY_F8"),  v8::Number::New(SDLK_F8));

	context->Global()->Set(v8::String::New("KEY_F9"),  v8::Number::New(SDLK_F9));

	context->Global()->Set(v8::String::New("KEY_F10"),  v8::Number::New(SDLK_F10));

	context->Global()->Set(v8::String::New("KEY_F11"),  v8::Number::New(SDLK_F11));

	context->Global()->Set(v8::String::New("KEY_F12"),  v8::Number::New(SDLK_F12));

	context->Global()->Set(v8::String::New("KEY_F13"),  v8::Number::New(SDLK_F13));

	context->Global()->Set(v8::String::New("KEY_F14"),  v8::Number::New(SDLK_F14));

	context->Global()->Set(v8::String::New("KEY_F15"),  v8::Number::New(SDLK_F15));

	context->Global()->Set(v8::String::New("KEY_TILDE"),  v8::Number::New(SDLK_BACKQUOTE));

	context->Global()->Set(v8::String::New("KEY_0"),  v8::Number::New(SDLK_0));

	context->Global()->Set(v8::String::New("KEY_1"),  v8::Number::New(SDLK_1));

	context->Global()->Set(v8::String::New("KEY_2"),  v8::Number::New(SDLK_2));

	context->Global()->Set(v8::String::New("KEY_3"),  v8::Number::New(SDLK_3));

	context->Global()->Set(v8::String::New("KEY_4"),  v8::Number::New(SDLK_4));

	context->Global()->Set(v8::String::New("KEY_5"),  v8::Number::New(SDLK_5));

	context->Global()->Set(v8::String::New("KEY_6"),  v8::Number::New(SDLK_6));

	context->Global()->Set(v8::String::New("KEY_7"),  v8::Number::New(SDLK_7));

	context->Global()->Set(v8::String::New("KEY_8"),  v8::Number::New(SDLK_8));

	context->Global()->Set(v8::String::New("KEY_9"),  v8::Number::New(SDLK_9));

	context->Global()->Set(v8::String::New("KEY_MINUS"),  v8::Number::New(SDLK_MINUS));

	context->Global()->Set(v8::String::New("KEY_EQUALS"),  v8::Number::New(SDLK_EQUALS));

	context->Global()->Set(v8::String::New("KEY_BACKSPACE"),  v8::Number::New(SDLK_BACKSPACE));

	context->Global()->Set(v8::String::New("KEY_SHIFT"),  v8::Number::New(SDLK_LSHIFT));

	context->Global()->Set(v8::String::New("KEY_RSHIFT"),  v8::Number::New(SDLK_RSHIFT));

	context->Global()->Set(v8::String::New("KEY_CAPSLOCK"),  v8::Number::New(SDLK_CAPSLOCK));

	context->Global()->Set(v8::String::New("KEY_NUMLOCK"),  v8::Number::New(SDLK_NUMLOCK));

	context->Global()->Set(v8::String::New("KEY_SCROLLOCK"),  v8::Number::New(SDLK_SCROLLOCK));

	context->Global()->Set(v8::String::New("KEY_CTRL"),  v8::Number::New(SDLK_LCTRL));

	context->Global()->Set(v8::String::New("KEY_RCTRL"),  v8::Number::New(SDLK_RCTRL));

	context->Global()->Set(v8::String::New("KEY_ALT"),  v8::Number::New(SDLK_LALT));

	context->Global()->Set(v8::String::New("KEY_RALT"),  v8::Number::New(SDLK_RALT));

	context->Global()->Set(v8::String::New("KEY_SPACE"),  v8::Number::New(SDLK_SPACE));

	context->Global()->Set(v8::String::New("KEY_OPENBRACE"),  v8::Number::New(SDLK_LEFTBRACKET));

	context->Global()->Set(v8::String::New("KEY_CLOSEBRACE"),  v8::Number::New(SDLK_RIGHTBRACKET));

	context->Global()->Set(v8::String::New("KEY_SEMICOLON"),  v8::Number::New(SDLK_SEMICOLON));

	context->Global()->Set(v8::String::New("KEY_APOSTROPHE"),  v8::Number::New(SDLK_QUOTE));

	context->Global()->Set(v8::String::New("KEY_COMMA"),  v8::Number::New(SDLK_COMMA));

	context->Global()->Set(v8::String::New("KEY_PERIOD"),  v8::Number::New(SDLK_PERIOD));

	context->Global()->Set(v8::String::New("KEY_SLASH"),  v8::Number::New(SDLK_SLASH));

	context->Global()->Set(v8::String::New("KEY_BACKSLASH"),  v8::Number::New(SDLK_BACKSLASH));

	context->Global()->Set(v8::String::New("KEY_INSERT"),  v8::Number::New(SDLK_INSERT));

	context->Global()->Set(v8::String::New("KEY_DELETE"),  v8::Number::New(SDLK_DELETE));

	context->Global()->Set(v8::String::New("KEY_HOME"),  v8::Number::New(SDLK_HOME));

	context->Global()->Set(v8::String::New("KEY_END"),  v8::Number::New(SDLK_END));

	context->Global()->Set(v8::String::New("KEY_PAGEUP"),  v8::Number::New(SDLK_PAGEUP));

	context->Global()->Set(v8::String::New("KEY_PAGEDOWN"),  v8::Number::New(SDLK_PAGEDOWN));

	context->Global()->Set(v8::String::New("KEY_UP"),  v8::Number::New(SDLK_UP));

	context->Global()->Set(v8::String::New("KEY_RIGHT"),  v8::Number::New(SDLK_RIGHT));

	context->Global()->Set(v8::String::New("KEY_DOWN"),  v8::Number::New(SDLK_DOWN));

	context->Global()->Set(v8::String::New("KEY_LEFT"),  v8::Number::New(SDLK_LEFT));

	context->Global()->Set(v8::String::New("KEY_NUM_0"),  v8::Number::New(SDLK_KP0));

	context->Global()->Set(v8::String::New("KEY_NUM_1"),  v8::Number::New(SDLK_KP1));

	context->Global()->Set(v8::String::New("KEY_NUM_2"),  v8::Number::New(SDLK_KP2));

	context->Global()->Set(v8::String::New("KEY_NUM_3"),  v8::Number::New(SDLK_KP3));

	context->Global()->Set(v8::String::New("KEY_NUM_4"),  v8::Number::New(SDLK_KP4));

	context->Global()->Set(v8::String::New("KEY_NUM_5"),  v8::Number::New(SDLK_KP5));

	context->Global()->Set(v8::String::New("KEY_NUM_6"),  v8::Number::New(SDLK_KP6));

	context->Global()->Set(v8::String::New("KEY_NUM_7"),  v8::Number::New(SDLK_KP7));

	context->Global()->Set(v8::String::New("KEY_NUM_8"),  v8::Number::New(SDLK_KP8));

	context->Global()->Set(v8::String::New("KEY_NUM_9"),  v8::Number::New(SDLK_KP9));

	context->Global()->Set(v8::String::New("KEY_NUM_PERIOD"),  v8::Number::New(SDLK_KP_PERIOD));

	context->Global()->Set(v8::String::New("KEY_NUM_DIVIDE"),  v8::Number::New(SDLK_KP_DIVIDE));

	context->Global()->Set(v8::String::New("KEY_NUM_MULTIPLY"),  v8::Number::New(SDLK_KP_MULTIPLY));

	context->Global()->Set(v8::String::New("KEY_NUM_MINUS"),  v8::Number::New(SDLK_KP_MINUS));

	context->Global()->Set(v8::String::New("KEY_NUM_EQUALS"),  v8::Number::New(SDLK_KP_EQUALS));

	context->Global()->Set(v8::String::New("KEY_RMETA"),  v8::Number::New(SDLK_RMETA));

	context->Global()->Set(v8::String::New("KEY_LMETA"),  v8::Number::New(SDLK_LMETA));

	context->Global()->Set(v8::String::New("KEY_RSUPER"),  v8::Number::New(SDLK_RSUPER));

	context->Global()->Set(v8::String::New("KEY_LSUPER"),  v8::Number::New(SDLK_LSUPER));

	context->Global()->Set(v8::String::New("KEY_BREAK"),  v8::Number::New(SDLK_BREAK));

	context->Global()->Set(v8::String::New("KEY_MENU"),  v8::Number::New(SDLK_MENU));

	context->Global()->Set(v8::String::New("KEY_POWER"),  v8::Number::New(SDLK_POWER));

	context->Global()->Set(v8::String::New("KEY_EURO"),  v8::Number::New(SDLK_EURO));
	//media/meta keys will usually show up as letter keys, for some reason.

}
