#include <sstream>
#include <iostream>
#include <fstream>
#include <string>
#include "opengame.h"
#include "graphic.h"
#define NO_BMP_FONT_DECL
#include "font.h"
#undef  NO_BMP_FONT_DECL
#include "SDL_V8.h"

using namespace std;

	v8::Persistent<v8::FunctionTemplate> Fonttempl;//  = *v8::FunctionTemplate::New();
	v8::Persistent<v8::ObjectTemplate> FontInsttempl;//= *Fonttempl->InstanceTemplate();
    v8::Persistent<v8::ObjectTemplate> Fontproto;//    = *Fonttempl->PrototypeTemplate();

void FontInit(){
    Fonttempl       = *v8::FunctionTemplate::New();
	FontInsttempl   = *Fonttempl->InstanceTemplate();
    Fontproto       = *Fonttempl->PrototypeTemplate();
    Fontproto->Set("drawText", v8::FunctionTemplate::New(TS_TTFdrawText));
	Fontproto->Set("drawZoomedText", v8::FunctionTemplate::New(TS_TTFdrawZoomedText));
	Fontproto->Set("getStringWidth", v8::FunctionTemplate::New(TS_TTFgetStringWidth));

}

void FontClose(){
    Fonttempl.Dispose();
	FontInsttempl.Dispose();
    Fontproto.Dispose();

}

void TS_TTFFontFinalizer(v8::Persistent<v8::Value> object, void* parameter) {
    TS_TTFFont* font = (TS_TTFFont*)parameter;
    delete font;
    object.Dispose();
    printf("A font object has been destroyed...?\n");
}

void TS_TTFFont::drawText(int x, int y, const char *text){
		//text_surface=SDL_DisplayFormat(TTF_RenderText_Solid(font,text,mask));
		TS_ShowSurface_Clear(TTF_RenderText_Solid(font,text,mask), x,y);
		//SDL_FreeSurface(text_surface);
}

void TS_TTFFont::drawZoomedText(int x, int y, float s, const char *text){
		//text_surface=TTF_RenderText_Solid(TTF_OpenFont(string(TS_dirs.font).append(fontname).c_str(), static_cast<int>(s*size)),text,mask);
		TS_ShowSurface_Clear(TTF_RenderText_Solid(TTF_OpenFont(string(TS_dirs.font).append(fontname).c_str(), static_cast<int>(s*size)),text,mask), x,y);
		//SDL_FreeSurface(text_surface);
}

int TS_TTFFont::getStringWidth(const char *text){
		text_surface=SDL_DisplayFormat(TTF_RenderText_Solid(font,text,mask));
		int s = text_surface->w;
		SDL_FreeSurface(text_surface);
		return s;
}

TS_TTFFont::TS_TTFFont(const char *f, int s){
	printf("%s\n", f);
	//const char *f2 = "DejaVuSans.ttf";
	//f = f2;
	fontname = f;
    SDL_RWops *fontfile = SDL_RWFromFile(f, "r");
    if(!fontfile){
        printf("\nerror opening file %s\n", f);
        printf("%s\n", SDL_GetError());
    }
    font=TTF_OpenFont(f, 10);
    if(!font) {
        printf("TTF_OpenFontRW: %s\n", TTF_GetError());
    }
    size = s;
    SDL_Color color = {255,255,255,128};
    mask = color;
    }

TS_TTFFont::TS_TTFFont(const char *f){
    printf(f);
    fontname = f;
    font = TTF_OpenFont(string(TS_dirs.font).append(f).c_str(), 10);
    size = 10;
    SDL_Color color = {255,255,255,128};
    mask = color;
}

v8::Handle<v8::Value> LoadTTFFont(const v8::Arguments& args) {
    if(args.Length()<1){
        return v8::ThrowException(v8::String::New("TS_LoadTTFFont Error: Called with no arguments."));
    }
    CHECK_ARG_STR(0, "TS_LoadTTFFont Error: Arg 0 is not a string.");
    if(args.Length()>1){
    CHECK_ARG_INT(1, "TS_LoadTTFFont Error: Arg 1 is not a number.");
    }
    v8::HandleScope loadfontscope;
	v8::Handle<v8::Value> external;
    TS_TTFFont *font = NULL;
    if (args[0]->IsExternal()) {
        external = v8::Local<v8::External>::Cast(args[0]);
    }
    else {
        int size = 10;
        v8::String::AsciiValue str(args[0]);
        const char *fontname = *str;
        if(!args[1]->IsUndefined()){
            size = args[1]->Int32Value();
        }
        font = new TS_TTFFont(string(TS_dirs.font).append(fontname).c_str(), size);
        external = v8::External::New(font);
    }
  	FontInsttempl->SetInternalFieldCount(1);
	v8::Handle<v8::Function> Fontctor = Fonttempl->GetFunction();
		//get class instance
	v8::Local<v8::Object> Fontobj = Fontctor->NewInstance();
		//Bridge the JS class seeds by setting the fields

    v8::Persistent<v8::Object> Pfontobj = v8::Persistent<v8::Object>::New(Fontobj);//colorctor->NewInstance();
    Fontobj.Clear();
    Pfontobj.MakeWeak(font, TS_TTFFontFinalizer);
	Pfontobj->SetInternalField(0, external);
    external.Clear();
    return loadfontscope.Close(Pfontobj);
}



TS_TTFFont::~TS_TTFFont(){
    TTF_CloseFont(font);
	SDL_FreeSurface(text_surface);
    delete [] fontname;
}


v8::Handle<v8::Value> GetFontname(v8::Local<v8::String> property, const v8::AccessorInfo &info) {
    v8::Local<v8::Object> self = info.Holder();
    v8::Local<v8::External> wrap = v8::Local<v8::External>::Cast(self->GetInternalField(0));
    void* ptr = wrap->Value();
    const char *value = static_cast<TS_TTFFont*>(ptr)->fontname;
    return v8::String::New(value);
}

void SetFontname(v8::Local<v8::String> property, v8::Local<v8::Value> value, const v8::AccessorInfo& info) {
				//READ ONLY!
				//Function required by V8 for both setters and getters.
}

v8::Handle<v8::Value> GetFontMask(v8::Local<v8::String> property, const v8::AccessorInfo &info){

return v8::Undefined();
}

void SetFontMask(v8::Local<v8::String> property, v8::Local<v8::Value> value, const v8::AccessorInfo& info) {
    v8::Local<v8::Object> self = info.Holder();
    v8::Local<v8::External> wrap = v8::Local<v8::External>::Cast(self->GetInternalField(0));
   // void* ptr = wrap->Value();
    self.Clear();
    wrap.Clear();
    //TS_Color* c = (TS_Color*)value->ToObject()->GetPointerFromInternalField(0);
    //SDL_Color color = {c->red, c->green,c->blue,c->alpha};
    //static_cast<TS_TTFFont*>(ptr)->mask = color;
}
/*
    const v8::AccessorInfo& info) {
    v8::Local<v8::Object> self = info.Holder();
    v8::Local<v8::External> wrap = v8::Local<v8::External>::Cast(self->GetInternalField(0));
    void* ptr = wrap->Value();
    static_cast<TS_Color*>(ptr)->blue = value->Int32Value();
*/
v8::Handle<v8::Value> GetFontsize(v8::Local<v8::String> property, const v8::AccessorInfo &info) {
    v8::Local<v8::Object> self = info.Holder();
    v8::Local<v8::External> wrap = v8::Local<v8::External>::Cast(self->GetInternalField(0));
    void* ptr = wrap->Value();
    int value = static_cast<TS_TTFFont*>(ptr)->size;
    self.Clear();
    wrap.Clear();
    return v8::Integer::New(value);
}

void SetFontsize(v8::Local<v8::String> property, v8::Local<v8::Value> value, const v8::AccessorInfo& info) {
    v8::Local<v8::Object> self = info.Holder();
    v8::Local<v8::External> wrap = v8::Local<v8::External>::Cast(self->GetInternalField(0));
    void* ptr = wrap->Value();
    if(value->IsNumber()){
        self.Clear();
        wrap.Clear();
        static_cast<TS_TTFFont*>(ptr)->size = value->Int32Value();
    }
}

v8::Handle<v8::Value> TS_TTFdrawText(const v8::Arguments& args) {
	if(args.Length()<3){
        return v8::ThrowException(v8::String::New("TS_drawText Error: called with less than 3 parameters!"));
    }
    CHECK_ARG_INT(0, "TS_drawText Error: could not parse arg 0 to number.")
    CHECK_ARG_INT(1, "TS_drawText Error: could not parse arg 1 to number.")

	int x = static_cast<int>(args[0]->v8::Value::NumberValue());
	int y = static_cast<int>(args[1]->v8::Value::NumberValue());
	v8::String::Utf8Value str(args[2]);
	//const char * z = args[2]->AsciiValue();
	v8::Local<v8::Object> self = args.Holder();
	//v8::Local<v8::External> wrap = v8::Local<v8::External>::Cast(self->GetInternalField(0));

	static_cast<TS_TTFFont*>(v8::Local<v8::External>::Cast(self->GetInternalField(0))->Value())->drawText(x,y,(const char*)(*str));
    //delete font;
    self.Clear();
    //wrap.Clear();

	return v8::Undefined();
}

v8::Handle<v8::Value> TS_TTFgetStringWidth(const v8::Arguments& args)
{
	v8::String::Utf8Value str(args[2]);
	v8::Local<v8::Object> self = args.Holder();
	v8::Local<v8::External> wrap = v8::Local<v8::External>::Cast(self->GetInternalField(0));
	void* ptr = wrap->Value();
	return v8::Number::New(static_cast<TS_TTFFont*>(ptr)->getStringWidth((const char*)(*str)));

}


v8::Handle<v8::Value> TS_TTFdrawZoomedText(const v8::Arguments& args)
{
	if(args.Length()<4)
	{
		printf("Error: TS_drawZoomedText called with less than 4 parameters!");
		return v8::Undefined();
	}
	//interpret the args so that they can be passes to drawText as a const char *
	int x = static_cast<int>(args[0]->v8::Value::NumberValue());
	int y = static_cast<int>(args[1]->v8::Value::NumberValue());
	float s = static_cast<float>(args[2]->v8::Value::NumberValue());
	v8::String::Utf8Value str(args[3]);
	v8::Local<v8::Object> self = args.Holder();
	v8::Local<v8::External> wrap = v8::Local<v8::External>::Cast(self->GetInternalField(0));
	void* ptr = wrap->Value();
	//use Function_A from Point pointer
	static_cast<TS_TTFFont*>(ptr)->drawZoomedText(x,y,s,(const char*)(*str));

	//	SDL_Surface *text_surface=TTF_RenderUTF8_Blended(static_cast<TS_TTFFont*>(ptr)->font,CSTR_text,static_cast<TS_TTFFont*>(ptr)->mask);
	//	TS_ShowSurface(text_surface, x,y);

	//return Integer::New(static_cast<Point*>(ptr)->x_);
	return v8::Undefined();
}
